
import os
import sys
import doctest

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
DIR_HOST = BASE_DIR.split('\\')[-1]
sys.path.append(BASE_DIR)

def all_doctest():
    for root, dirs, files in os.walk(BASE_DIR):
        for f in files:
            name, ext = os.path.splitext(f)
            if ext == '.py':
                root = root.replace(BASE_DIR, '').replace('/', '.').replace('\\', '.')
                import_name = DIR_HOST + root + '.' + name
                importpy = __import__(import_name, globals(), locals(), name, 0)
                doctest.testmod(importpy)




# -*- coding:utf-8 -*-

from lawes.db.models.query import QuerySet
from lawes.utils import six
from lawes.db.models.options import Options
import json
import datetime
import copy
from bson.objectid import ObjectId
from pymongo.results import InsertOneResult
from lawes.db.models.fields import AutoField

class ModelBase(type):

    def __new__(cls, name, bases, attrs):
        """  changed Field to true type
        """
        super_new = super(ModelBase, cls).__new__
        # if it is not the ModelBase , return
        parents = [b for b in bases if isinstance(b, ModelBase)]
        if not parents:
            return super_new(cls, name, bases, attrs)

        # Create the class.
        # new_class = super_new(cls, name, bases, attrs)
        # module = attrs.pop('__module__')
        new_class = super_new(cls, name, bases, attrs)
        meta = attrs.pop('Meta', None)
        if not meta:
            meta = getattr(new_class, 'Meta', None)
        # db_table is the collection name in mongodb
        db_table = new_class.__module__.split('.')[-1] + '_' + new_class.__name__.lower()
        kwargs = {"db_table": db_table, 'module_name': new_class.__name__.lower()}
        new_class._meta = Options(meta, **kwargs)

        # set the local_fields at Options: {'name': lawes.db.models.fields.CharField}
        for obj_name, obj in attrs.items():
            if hasattr(obj, 'contribute_to_class'):
                new_class._meta.add_field(obj_name=obj_name, obj=obj)

        # create the objects
        new_class.objects = QuerySet(model=new_class)

        return new_class


class Model(six.with_metaclass(ModelBase)):

    pk_attname = '_id'
    save_fields = []

    def __init__(self, *args, **kwargs):
        # set the real value to the model
        for obj_name in self._meta.local_fields:
            obj = self._meta.local_fields[obj_name]
            setattr(self, obj_name, obj.value)
        super(Model, self).__init__()

    def __setattr__(self, key, value):
        super(Model, self).__setattr__(key, value)
        if hasattr(self, '_id'):
            if not key in self.save_fields:
                self.save_fields.append(key)

    def save(self):
        local_fields = {}
        for field in self._meta.local_fields:
            if isinstance(self._meta.local_fields[field], AutoField):
                if getattr(self, field) is None:
                    local_fields[field] = self._meta.local_fields[field]
        auto_results = self.objects.save_auto_field(local_fields=local_fields)
        for auto_field in auto_results:
            setattr(self, auto_field, auto_results[auto_field])
        self._save_table()

    def _save_table(self):
        pk_val = self._get_pk_val()
        # pk_set is true for updating, pk_set is true for inserting
        pk_set = pk_val is not None
        data = self.to_dict(fields='save_fields') if pk_set else self.to_dict()
        # the format must be correct
        checking_keys = set(self._meta.local_fields) & set(data)
        for key in checking_keys:
            self._meta.local_fields[key].check_type(value=data[key])
        if pk_set:
            self._do_update(data=data)
        else:
            result = self._do_insert(data=data)
            setattr(self, self.pk_attname, result)

    def _get_pk_val(self):
        """ get _id: None: INSERT; not None: UPDATE
        """
        if not hasattr(self, self.pk_attname):
            return None
        else:
            return getattr(self, self.pk_attname)

    @classmethod
    def _do_insert(cls, data):
        """ 向 mongodb 插入数据
        """
        return cls.objects._insert(data=data)

    @classmethod
    def _do_update(cls, data):
        """ doing update in mongodb
        """
        return cls.objects._update(data=data)

    def to_dict(self, fields=''):
        """ if the values of the fields is 'save_fields' , only change the fields parts
        """
        if fields == 'save_fields':
            fields_type = self.save_fields
        else:
            fields_type = self._meta.local_fields
        result = {}

        for field in fields_type:
            if hasattr(self, field):
                value = getattr(self, field)
                result[field] = value

        if hasattr(self, '_id'):
            if isinstance(self._id, InsertOneResult):
                result['_id'] = self._id.inserted_id
            else:
                result['_id'] = self._id
        return result

    def to_json(self):
        result = copy.deepcopy(self.to_dict())
        to_str_list = (datetime.datetime, ObjectId,)
        for r in result:
            for to_obj_type in to_str_list:
                if isinstance(result[r], to_obj_type):
                    result[r] = str(result[r])
        return result

    def to_dict_format(self):
        result = self.to_json()
        result = json.dumps(result, indent=4, ensure_ascii=False, sort_keys=True)
        return result

    def to_obj(self, data={}):

        for field in data:
            setattr(self, field, data[field])
        for field in self._meta.local_fields:
            if field in data:
                value = data[field]
            else:
                value = self._meta.local_fields[field].value
            setattr(self, field, value)
        if '_id' in data:
            self._id = data['_id']
        return self

    def delete(self):
        return self.objects._remove(_id=self._id)

    def clean_auto_field(self):
        self.objects.clean_auto_field(local_fields=self._meta.local_fields)




# -*- coding:utf-8 -*-

from lawes.db.models.lookups import RegisterLookupMixin
import datetime
from lawes.core.exceptions import ValidationError
from lawes.core.exceptions import DefaultError

class CheckTypeNone(object):
    pass

class Field(RegisterLookupMixin):
    """ 用于标识是否需要转换和存储的字段
    """
    contribute_to_class = None
    error_message = "{value} value has an invalid format. It must be in {value_type} format."
    # 可以不用设置 default, 获得设置成空
    default_can_set_null = False

    def __init__(self, default=None, db_index=False, unique=False):
        if not hasattr(self, 'default') and default is None:
            raise DefaultError('This Field needs default')
        if not default is None:
            self.default = default
        # self.value = default
        self.db_index = db_index
        self.unique = unique

    @property
    def value(self):
        if callable(self.default):
            default = self.default()
        else:
            default = self.default
        return default

    def check_type(self, value=CheckTypeNone()):
        """ 检测子类的类型是否正确
        """
        check_value = value if not isinstance(value, CheckTypeNone) else self.value
        if self.default_can_set_null is True and check_value is None:
            return True
        if not isinstance(check_value, self.field_type):
            raise ValidationError(message=self.error_message,params={'value': check_value, 'value_type': str(self.field_type)},)
        return True

class CharField(Field):

    field_type = str
    default = ''

    def __init__(self, *args, **kwargs):
        super(CharField, self).__init__(*args, **kwargs)

class IntegerField(Field):

    field_type = int
    default = 0

    def __init__(self, *args, **kwargs):
        super(IntegerField, self).__init__(*args, **kwargs)

class FloatField(Field):

    field_type = float
    default = 0.0

    def __init__(self, *args, **kwargs):
        super(FloatField, self).__init__(*args, **kwargs)

class DateTimeField(Field):

    field_type = datetime.datetime
    default_can_set_null = True
    default = None

    def __init__(self, *args, **kwargs):
        super(DateTimeField, self).__init__(*args, **kwargs)

class BooleanField(Field):

    field_type = bool
    default = False

    def __init__(self, *args, **kwargs):
        super(BooleanField, self).__init__(*args, **kwargs)


class ArrayField(Field):

    field_type = list
    default = []

    def __init__(self, *args, **kwargs):
        super(ArrayField, self).__init__(*args, **kwargs)


class HStoreField(Field):

    field_type = dict
    default = {}

    def __init__(self, *args, **kwargs):
        super(HStoreField, self).__init__(*args, **kwargs)


class AutoField(Field):

    field_type = int
    default_can_set_null = True
    default = None
    start = None

    def __init__(self, *args, **kwargs):
        if 'start' in kwargs:
            self.start = kwargs.pop('start')
        super(AutoField, self).__init__(*args, **kwargs)




import unittest
from lawes.test.test_db_models_fields import *
from lawes.test.test_db_models_base import *
from lawes.test.test_db_models_query_utils_q import *
from lawes.test.all_doctest import all_doctest

if __name__ == "__main__":
    all_doctest()
    unittest.main()



import copy

class Node(object):

    default = 'single'

    def __init__(self, connector=None, values=[]):
        self.q_left = None
        self.q_right = None
        self.connector = connector or self.default
        self.values = values[:] if values else []       # [( 'name', 'lawes' ), ( 'age': 18 )]

    def __str__(self):
        if self.q_left and self.q_left:
            return '[ %s %s %s ]' % (str(self.q_left), self.connector, str(self.q_right))
        else:
            return "( '%s', %s)" % (self.connector, self.values)

    def __bool__(self):
        if not self.q_left and not self.q_right and not self.values:
            return False
        return True

    def add(self, q_left, q_right, conn_type):
        if q_left and q_right:
            self.q_left = q_left
            self.q_right = q_right
            self.connector = conn_type
        elif not q_left and not q_right:
            pass
        else:
            if conn_type == self.AND:
                self = q_left or q_right
            elif conn_type == self.OR:
                if not q_left:
                    self = q_left
                else:
                    self = q_right
        return self




from lawes.db import models

conf_dict = {'mongo_uri': 'mongodb://127.0.0.1:27017/', 'db_name': 'testdb'}
models.setup(conf=conf_dict)

class Fruit(models.Model):
    class Meta:
        db_table = 'fruit'

    name = models.CharField(default='')
    color = models.CharField(default='')
    sid = models.AutoField()
    nid = models.AutoField(start=1000000000)



from lawes.utils import tree
from lawes.utils import six
from bson.objectid import ObjectId

class Q(tree.Node):
    """
    Encapsulates filters as objects that can then be combined logically (using
    `&` and `|`).
    """
    AND = '$and'
    OR = '$or'

    def __init__(self, *args, **kwargs):
        if '_id' in kwargs :
            kwargs['_id'] = ObjectId(kwargs['_id'])
        super(Q, self).__init__(values=list(six.iteritems(kwargs)))

    def _combine(self, other, conn):
        if not isinstance(other, Q):
            raise TypeError(other)
        obj = type(self)()
        obj = obj.add(self, other, conn)
        return obj

    def __or__(self, other):
        return self._combine(other, self.OR)

    def __and__(self, other):
        return self._combine(other, self.AND)




import unittest
from lawes.test import Fruit
from lawes.db.models import Q
from lawes.db.models.sql import Query

class TestBdModelsQueryUtilsQ(unittest.TestCase):

    def setUp(self):
        self.clearall()

    def tearDown(self):
        self.clearall()

    def clearall(self):
        for test_obj in Fruit.objects.filter():
            test_obj.delete()

    def test_q(self):
        query = Q()
        result = Fruit.objects.filter(query)
        self.assertEqual(result.query.as_sql(), {})
        query = query & Q(name='lawes')
        result = Fruit.objects.filter(query)
        self.assertEqual(result.query.as_sql(), {'name': 'lawes'})
        query = query | Q()
        result = Fruit.objects.filter(query)
        self.assertEqual(result.query.as_sql(), {})
        query = query & Q(name='lawes') & Q(sex=1)
        result = Fruit.objects.filter(query)
        query_expected = {'$and': [{'name': 'lawes'}, {'sex': 1}]}
        self.assertEqual(result.query.as_sql(), query_expected)
        query_other = Q(address='lawes street1') | Q(age__gt=2)
        result = Fruit.objects.filter(query_other)
        query_other_expected = {'$or': [{'address': 'lawes street1'}, {'age': {'$gt': 2}}]}
        self.assertEqual(result.query.as_sql(), query_other_expected)
        query_combine = query & query_other
        result = Fruit.objects.filter(query_combine)
        self.assertEqual(result.query.as_sql(), {'$and': [query_expected, query_other_expected]})
        query = Q(name_text__search='name')
        query = query | Q(address_text__search='address')
        query_result = {'$or': [{'name': {'$regex': 'n.*a.*m.*e', '$options': 'si'}}, {'address': {'$regex': 'a.*d.*d.*r.*e.*s.*s', '$options': 'si'}}]}
        result = Fruit.objects.filter(query)
        self.assertEqual(result.query.as_sql(), query_result)

    # TODO all of them




import unittest
from lawes.test import Fruit
from lawes.db.models import Q

class TestBdModelsBase(unittest.TestCase):

    def setUp(self):
        self.clearall()

    def tearDown(self):
        self.clearall()

    def clearall(self):
        for test_obj in Fruit.objects.filter():
            test_obj.delete()

    def test_delete(self):
        self.clearall()
        for test_obj in Fruit.objects.filter():
            self.fail('delete does not work!')

    # TODO all of them




class Query(object):
    """
    A single SQL query.
    """
    def __init__(self, model):
        self.model = model
        self.q_object = None                        # lawes.db.models import Q
        self.order_by_query = ()                    # using for Model.objects.order_by(filter_query)
        self.skip = None                            # using for Model.objects.skip(skip)#
        self.limit = None                           # using for Model.objects.limit(limit)#

    def add_q(self, q_object):
        """
        :param q_object: lawes.db.models import Q
        :return:
        """
        if not self.q_object:
            self.q_object = q_object
        else:
            self.q_object = self.q_object & q_object

    def filter_comparsion(self, query):
        """ if found __gt, __gte, __lt, __lte, __ne in query, change to "$gt", "$gte", "$lt", "$lte", "$ne"
        >>> query = {"1__gt": 1,"2__gte": 2,"3__lt": 3,"4__lte": 4,"5__ne": 5, '6': 6, '7_text__search': '77' }
        >>> Query(None).filter_comparsion(query=query)
        {'1': {'$gt': 1}, '2': {'$gte': 2}, '3': {'$lt': 3}, '4': {'$lte': 4}, '5': {'$ne': 5}, '6': 6, '7': {'$regex': '7.*7', '$options': 'si'}}
        """
        c_query = {}
        match_tuple = ( '__gt', '__gte', '__lt', '__lte', '__ne', )
        for qkey in query:
            origin = True
            for match_key in match_tuple:
                if qkey.endswith(match_key):
                    match_key_to = match_key.replace('__', '$')
                    startpart, endpart = qkey.split(match_key)
                    c_query[startpart] = { match_key_to: query[qkey] }
                    origin = False
            if qkey.endswith('_text__search'):
                startpart, endpart = qkey.split('_text__search')
                if not isinstance(query[qkey], str):
                    raise TypeError('Can not _text__search with %s, only string' % ( type(query[qkey] ) ) )
                c_query[startpart] = {'$regex': '.*'.join(query[qkey]), '$options': 'si'}
                origin = False
            if origin is True:
                c_query[qkey] = query[qkey]
        return c_query

    def _as_sql(self, q_object):
        """
        >>> from lawes.db.models import Q
        >>> query = Q(name='lawes')
        >>> obj = Query(None)
        >>> obj._as_sql(q_object=query)
        {'name': 'lawes'}
        >>> query = query & Q(sex=1)
        >>> obj._as_sql(q_object=query)
        {'$and': [{'name': 'lawes'}, {'sex': 1}]}
        >>> query_other = Q(address='lawes street1') | Q(age__gt=2)
        >>> obj._as_sql(q_object=query_other)
        {'$or': [{'address': 'lawes street1'}, {'age': {'$gt': 2}}]}
        >>> obj._as_sql(q_object=query & query_other)
        {'$and': [{'$and': [{'name': 'lawes'}, {'sex': 1}]}, {'$or': [{'address': 'lawes street1'}, {'age': {'$gt': 2}}]}]}
        """
        filter_query = {}
        if q_object.q_right is None and q_object.q_left is None:
            if q_object.values:
                filter_query = self.filter_comparsion(dict(q_object.values))
        else:
            filter_query[q_object.connector] = [self._as_sql(q_object.q_left), self._as_sql(q_object.q_right) ]
        return filter_query

    def as_sql(self):
        filter_query = self._as_sql(self.q_object)
        return filter_query

    def execute_sql(self, collection):
        filter_query = self.as_sql()
        multi_data = collection.find(filter_query)
        # order by query
        if self.order_by_query:
            multi_data = multi_data.sort(*self.order_by_query)
        if not self.skip is None:
            multi_data = multi_data.skip(self.skip)
        if not self.limit is None:
            multi_data = multi_data.limit(self.limit)
        return multi_data



import unittest
from lawes.test import Fruit
from lawes.db.models.fields import AutoField

class TestBdModelsFields(unittest.TestCase):

    def setUp(self):
        self.clearall()

    def tearDown(self):
        self.clearall()

    def clearall(self):
        for test_obj in Fruit.objects.filter():
            test_obj.delete()
        Fruit().clean_auto_field()

    def test_autofield(self):
        fruit = Fruit()
        fruit.save()
        self.assertEqual(fruit.nid, 1000000001)
        fruit.save()
        self.assertEqual(fruit.sid, 1)

    # TODO all of them




# -*- coding:utf-8 -*-

class ValidationError(Exception):

    def __init__(self, message, code=None, params=None):

        super(ValidationError, self).__init__(message, code, params)
        self.message = message
        self.params = params

    def __str__(self):
        return self.message.format(**self.params)


class ImproperlyConfigured(Exception):
    """Lawes is somehow improperly configured"""
    pass


class MultipleObjectsReturned(Exception):
    """The query returned multiple objects when only one was expected."""
    pass


class DoesNotExist(Exception):
    """The query returned None objects when only one was expected."""
    pass


class DoesNotExist(Exception):
    """The query returned None objects when only one was expected."""
    pass


class UniqueError(Exception):
    pass


class MongoClientError(Exception):
    pass


class DefaultError(Exception):
    pass


# -*- coding:utf-8 -*-

from pymongo import MongoClient
from lawes.core.exceptions import MultipleObjectsReturned
from lawes.core.exceptions import DoesNotExist
from lawes.core.exceptions import UniqueError
from lawes.core.exceptions import MongoClientError
from pymongo.errors import OperationFailure
from bson.objectid import ObjectId
from lawes.db.models.query_utils import Q
from lawes.db.models import sql
from lawes.db.models.fields import AutoField
import copy

CONF_RAESE = """
The correct formal is:
from lawes.db import models
models.setup(conf={'mongo_uri': 'mongodb://127.0.0.1:27017/test', 'db_name': 'testindex'})
self._mongo: %s , self._db: %s
"""

class ConfigQuerySet(object):

    def __init__(self):
        self.mongo = None
        self.db_name = ''

    def _setup(self, conf):
        """ 设置mongodb的连接方式
        """
        if self.mongo:
            return
        if self.db_name:
            return
        if not 'db_name' in conf:
            raise MongoClientError(CONF_RAESE % ('', ''))
        if not 'mongo_uri' in conf:
            raise MongoClientError(CONF_RAESE % ('', ''))
        self.mongo = MongoClient(conf['mongo_uri'])
        self.db_name = conf['db_name'].lower()

# init the MongoClient
configqueryset = ConfigQuerySet()

class QuerySet(object):

    def __init__(self, model=None, query=None):

        self.query_flag = False                    # set the query_flag to True after filer, order_by , that the methon only return QuerySet
        self.model = model
        self._mongo = configqueryset.mongo
        self._db = configqueryset.db_name           # the name of the db
        self.db_table = model._meta.db_table        # the name of the collection
        if not self._mongo or not self._db:
            raise MongoClientError(CONF_RAESE % (str(self._mongo), str(self._db)))
        self._collection = getattr(self._mongo[self._db], self.db_table)
        self.query = query or sql.Query(model)

    def __iter__(self):
        for data in self._fetch_all():
            yield data

    def __getitem__(self, item):
        if isinstance(item, int):
            self.query.limit = item
            self.query.skip = item
        elif isinstance(item, slice):
            start = item.start or 0
            self.query.skip = start
            self.query.limit = item.stop - start
        if isinstance(item, int):
            for iter_item in self.__iter__():
                return iter_item
        else:
            return self.__iter__()

    def __bool__(self):
        return bool(self.count())

    def __len__(self):
        return self.count()

    def count(self):
        return self.query.execute_sql(collection=self._collection).count()

    def _clone(self):
        if self.query_flag is False:
            obj = QuerySet(model=self.model)
        else:
            obj = self
        return obj

    def filter(self, *args, **kwargs):
        return self._filter_or_exclude(False, *args, **kwargs)

    def exclude(self, *args, **kwargs):
        return self._filter_or_exclude(True, *args, **kwargs)

    def _filter_or_exclude(self, negate, *args, **kwargs):
        if args and kwargs:
            return self
        clone = self._clone()
        clone.query_flag = True
        if negate:
            query = ~Q(*args, **kwargs) # TODO This has not QAed yet
        else:
            query = Q(*args, **kwargs)
        for arg in args:
            query = arg & query
        clone.query.add_q(query)
        return clone

    def _fetch_all(self, originally=False):
        """ run the sql actually
        :return:
        """
        multi_data = self.query.execute_sql(collection=self._collection)
        if originally is True:
            for data in multi_data:
                yield data
        else:
            for data in multi_data:
                obj = self.model()
                obj = obj.to_obj(data=data)
                yield obj

    def _insert(self, data):
        """
        Inserts a new record for the given model. This provides an interface to
        the InsertQuery class and is how Model.save() is implemented.
        """
        return self._collection.insert_one(data)

    def _update(self, data):
        """
        Inserts a new record for the given model. This provides an interface to
        the InsertQuery class and is how Model.save() is implemented.
        """
        mongodb_id = data.pop('_id')
        if not data:
            return None
        return self._collection.update({'_id': mongodb_id}, {'$set': data}, upsert=True)

    def init_index(self):
        """  create the index_1
        :param db_indexs: {'name': {'unique': False}}
        :return:
        """
        db_indexs = self.model._meta.db_indexs
        try:
            old_index = self._collection.index_information()
        except OperationFailure:
            old_index = {}

        for attr in db_indexs:
            unique = db_indexs[attr].get('unique', False)
            if not attr + '_1' in old_index:
                if unique is True:
                    self._collection.ensure_index(attr,unique=True,background=True)
                else:
                    self._collection.ensure_index(attr,background=True)
            elif unique is True:
                if not 'unique' in old_index[attr + '_1']:
                    self._collection.drop_index(attr + '_1')
                    self._collection.ensure_index(attr,unique=True,background=True)

    def get(self, *args, **kwargs):
        """
        Performs the query and returns a single object matching the given
        keyword arguments.
        """
        # self.filter_query.update(kwargs)
        # data = self._collection.find(self.filter_query)
        self = self._filter_or_exclude(False, *args, **kwargs)
        data = self.query.execute_sql(collection=self._collection)
        num = data.count()
        if num == 1:
            obj = self.model()
            obj = obj.to_obj(data=data[0])
            return obj
        if not num:
            raise DoesNotExist(
                "%s matching query does not exist." %
                self.query.as_sql()
            )
        raise MultipleObjectsReturned(
            "findone  %s returned more than one -- it returned %s!" %
            (self.query.as_sql(), num)
        )

    def get_or_create(self, **kwargs):
        """
        :param kwargs:  name='x'
        :return:  a tuple of (object, created), where created is a boolean
        specifying whether an object was created.
        """
        created = False
        try:
            index_information = self._collection.index_information()
        except OperationFailure:
            index_information = {}

        kwargs_keys = set(kwargs.keys())
        index_keys = [ filed for filed in index_information if index_information[filed].get('unique', False)]
        index_keys = [ index_key.replace('_1', '').replace('_-1', '') for index_key in index_keys ]
        need_index_set = kwargs_keys - set(index_keys)
        if need_index_set:
            raise UniqueError('UNIQUE constraint failed: %s: %s, please do collection.ensure_index: Model.objects.init_index()' % (
            self.db_table, str(need_index_set)))
        # self.filter_query.update(kwargs)
        # data = self._collection.find(self.filter_query)
        self = self._filter_or_exclude(False, **kwargs)
        data = self.query.execute_sql(collection=self._collection)
        num = data.count()
        if num == 1:
            to_obj_data = data[0]
        elif not num:
            to_obj_data = kwargs
            created = True
        else:
            raise MultipleObjectsReturned(
                "findone  %s returned more than one -- it returned %s!" %
                (self.query.as_sql(), num)
            )
        obj = self.model()
        obj = obj.to_obj(data=to_obj_data)
        if not num:
            obj.save()
        return obj, created

    def order_by(self, *field_names):
        field_names = field_names[0]
        if '-' in field_names:
            field_names = field_names.replace('-', '')
            order_index = -1
        else:
            order_index = 1
        self.query.order_by_query = (field_names, order_index)
        return self

    def delete(self):
        """
        Deletes the records in the current QuerySet.
        """
        for data in self._fetch_all(originally=True):
            self._remove(_id=str(data['_id']))

    def _remove(self, _id):
        remove_dict = {'_id': ObjectId(_id)}
        self._collection.delete_many(remove_dict)

    def get_auto_collection(self, field_name):
        auto_collection = getattr(self._mongo[self._db], self.db_table + '_auto_%s' % (field_name))
        return auto_collection

    def save_auto_field(self, local_fields=[]):
        data = {}
        local_fields = copy.deepcopy(local_fields)
        for field_name in local_fields:
            auto_collection = self.get_auto_collection(field_name=field_name)
            start = local_fields[field_name].start
            if not start is None:
                filter_dict = { 'key': field_name , 'autoid': {'$lt': start} }
                update_dict = { '$set' :{'autoid': start } }
                auto_collection.update_one(filter_dict, update_dict, upsert=True)
            auto_infos = auto_collection.find_one_and_update(filter={'key': field_name}, update={'$inc': {'autoid': 1}}, new=True, upsert=True)
            data[field_name] = auto_infos['autoid']
        return data

    def clean_auto_field(self, local_fields=[]):
        local_fields = copy.deepcopy(local_fields)
        for field_name in local_fields:
            if isinstance(local_fields[field_name], AutoField):
                auto_collection = self.get_auto_collection(field_name=field_name)
                auto_collection.delete_many({})